@@ -4,6 +4,9 @@ from django.utils.translation import ugettext_lazy as _
 
 import logging
 
+# Allow local testing of Sentry even if DEBUG is enabled
+DEBUG = getattr(settings, 'DEBUG', False) and not getattr(settings, 'SENTRY_TESTING', False)
+
 CATCH_404_ERRORS = getattr(settings, 'SENTRY_CATCH_404_ERRORS', False)
 
 DATABASE_USING = getattr(settings, 'SENTRY_DATABASE_USING', None)
@@ -31,4 +34,8 @@ LOG_LEVELS = (
 )
 
 # This should be the full URL to sentries store view
-REMOTE_URL = getattr(settings, 'SENTRY_REMOTE_URL', None)
\ No newline at end of file
+REMOTE_URL = getattr(settings, 'SENTRY_REMOTE_URL', None)
+
+REMOTE_TIMEOUT = getattr(settings, 'SENTRY_REMOTE_TIMEOUT', 5)
+
+ADMINS = getattr(settings, 'SENTRY_ADMINS', [])
\ No newline at end of file
