@@ -5,10 +5,13 @@ try:
     import cPickle as pickle
 except ImportError:
     import pickle
+import getpass
 import logging
 import sys
 import threading
 
+from django.contrib.auth.models import User
+from django.core import mail
 from django.core.handlers.wsgi import WSGIRequest, WSGIHandler
 from django.core.urlresolvers import reverse
 from django.core.signals import got_request_exception
@@ -147,24 +150,24 @@ class SentryTestCase(TestCase):
         self.setUpHandler()
 
         logger.error('This is a test error')
-        cur = (Message.objects.count(), GroupedMessage.objects.count())
-        self.assertEquals(cur, (1, 1), 'Assumed logs failed to save. %s' % (cur,))
-        last = Message.objects.all().order_by('-id')[0:1].get()
+        self.assertEquals(Message.objects.count(), 1)
+        self.assertEquals(GroupedMessage.objects.count(), 1)
+        last = Message.objects.get()
         self.assertEquals(last.logger, 'root')
         self.assertEquals(last.level, logging.ERROR)
         self.assertEquals(last.message, 'This is a test error')
 
         logger.warning('This is a test warning')
-        cur = (Message.objects.count(), GroupedMessage.objects.count())
-        self.assertEquals(cur, (2, 2), 'Assumed logs failed to save. %s' % (cur,))
+        self.assertEquals(Message.objects.count(), 2)
+        self.assertEquals(GroupedMessage.objects.count(), 2)
         last = Message.objects.all().order_by('-id')[0:1].get()
         self.assertEquals(last.logger, 'root')
         self.assertEquals(last.level, logging.WARNING)
         self.assertEquals(last.message, 'This is a test warning')
         
         logger.error('This is a test error')
-        cur = (Message.objects.count(), GroupedMessage.objects.count())
-        self.assertEquals(cur, (3, 2), 'Assumed logs failed to save. %s' % (cur,))
+        self.assertEquals(Message.objects.count(), 3)
+        self.assertEquals(GroupedMessage.objects.count(), 2)
         last = Message.objects.all().order_by('-id')[0:1].get()
         self.assertEquals(last.logger, 'root')
         self.assertEquals(last.level, logging.ERROR)
@@ -172,16 +175,16 @@ class SentryTestCase(TestCase):
     
         logger = logging.getLogger('test')
         logger.info('This is a test info')
-        cur = (Message.objects.count(), GroupedMessage.objects.count())
-        self.assertEquals(cur, (4, 3), 'Assumed logs failed to save. %s' % (cur,))
+        self.assertEquals(Message.objects.count(), 4)
+        self.assertEquals(GroupedMessage.objects.count(), 3)
         last = Message.objects.all().order_by('-id')[0:1].get()
         self.assertEquals(last.logger, 'test')
         self.assertEquals(last.level, logging.INFO)
         self.assertEquals(last.message, 'This is a test info')
         
         logger.info('This is a test info with a url', extra=dict(url='http://example.com'))
-        cur = (Message.objects.count(), GroupedMessage.objects.count())
-        self.assertEquals(cur, (5, 4), 'Assumed logs failed to save. %s' % (cur,))
+        self.assertEquals(Message.objects.count(), 5)
+        self.assertEquals(GroupedMessage.objects.count(), 4)
         last = Message.objects.all().order_by('-id')[0:1].get()
         self.assertEquals(last.url, 'http://example.com')
         
@@ -189,8 +192,8 @@ class SentryTestCase(TestCase):
             raise ValueError('This is a test ValueError')
         except ValueError:
             logger.info('This is a test info with an exception', exc_info=sys.exc_info())
-            cur = (Message.objects.count(), GroupedMessage.objects.count())
-            self.assertEquals(cur, (6, 5), 'Assumed logs failed to save. %s' % (cur,))
+            self.assertEquals(Message.objects.count(), 6)
+            self.assertEquals(GroupedMessage.objects.count(), 5)
             last = Message.objects.all().order_by('-id')[0:1].get()
             self.assertEquals(last.class_name, 'ValueError')
             self.assertEquals(last.message, 'This is a test info with an exception')
@@ -211,9 +214,9 @@ class SentryTestCase(TestCase):
         else:
             self.fail('Unable to create `Message` entry.')
         
-        cur = (Message.objects.count(), GroupedMessage.objects.count())
-        self.assertEquals(cur, (1, 1), 'Assumed logs failed to save. %s' % (cur,))
-        last = Message.objects.all().order_by('-id')[0:1].get()
+        self.assertEquals(Message.objects.count(), 1)
+        self.assertEquals(GroupedMessage.objects.count(), 1)
+        last = Message.objects.get()
         self.assertEquals(last.logger, 'root')
         self.assertEquals(last.class_name, 'DoesNotExist')
         self.assertEquals(last.level, logging.ERROR)
@@ -236,8 +239,8 @@ class SentryTestCase(TestCase):
             self.fail('Unable to create `Message` entry.')
 
         
-        cur = (Message.objects.count(), GroupedMessage.objects.count())
-        self.assertEquals(cur, (2, 2), 'Assumed logs failed to save. %s' % (cur,))
+        self.assertEquals(Message.objects.count(), 2)
+        self.assertEquals(GroupedMessage.objects.count(), 2)
         last = Message.objects.all().order_by('-id')[0:1].get()
         self.assertEquals(last.logger, 'root')
         self.assertEquals(last.class_name, 'DoesNotExist')
@@ -246,8 +249,8 @@ class SentryTestCase(TestCase):
         
         SentryClient.create_from_text('This is an error', level=logging.DEBUG)
         
-        cur = (Message.objects.count(), GroupedMessage.objects.count())
-        self.assertEquals(cur, (3, 3), 'Assumed logs failed to save. %s' % (cur,))
+        self.assertEquals(Message.objects.count(), 3)
+        self.assertEquals(GroupedMessage.objects.count(), 3)
         last = Message.objects.all().order_by('-id')[0:1].get()
         self.assertEquals(last.logger, 'root')
         self.assertEquals(last.level, logging.DEBUG)
@@ -263,9 +266,9 @@ class SentryTestCase(TestCase):
         else:
             self.fail('Unable to create `Message` entry.')
             
-        cur = (Message.objects.count(), GroupedMessage.objects.count())
-        self.assertEquals(cur, (1, 1), 'Assumed logs failed to save. %s' % (cur,))
-        last = Message.objects.all().order_by('-id')[0:1].get()
+        self.assertEquals(Message.objects.count(), 1)
+        self.assertEquals(GroupedMessage.objects.count(), 1)
+        last = Message.objects.get()
         self.assertEquals(last.logger, 'root')
         self.assertEquals(last.class_name, 'DoesNotExist')
         self.assertEquals(last.level, logging.ERROR)
@@ -344,9 +347,9 @@ class SentryTestCase(TestCase):
         else:
             self.fail('Expected an exception.')
         
-        cur = (Message.objects.count(), GroupedMessage.objects.count())
-        self.assertEquals(cur, (1, 1), 'Assumed logs failed to save. %s' % (cur,))
-        last = Message.objects.all().order_by('-id')[0:1].get()
+        self.assertEquals(Message.objects.count(), 1)
+        self.assertEquals(GroupedMessage.objects.count(), 1)
+        last = Message.objects.get()
         self.assertEquals(last.logger, 'root')
         self.assertEquals(last.class_name, 'DoesNotExist')
         self.assertEquals(last.level, logging.ERROR)
@@ -366,8 +369,8 @@ class SentryTestCase(TestCase):
         else:
             self.fail('Expected an exception.')
         
-        cur = (Message.objects.count(), GroupedMessage.objects.count())
-        self.assertEquals(cur, (1, 1), 'Assumed logs failed to save. %s' % (cur,))
+        self.assertEquals(Message.objects.count(), 1)
+        self.assertEquals(GroupedMessage.objects.count(), 1)
         
         settings.USE_LOGGING = False
     
@@ -393,9 +396,9 @@ class SentryTestCase(TestCase):
         else:
             self.fail('Expected an exception.')
             
-        cur = (Message.objects.count(), GroupedMessage.objects.count())
-        self.assertEquals(cur, (1, 1), 'Assumed logs failed to save. %s' % (cur,))
-        last = Message.objects.all().order_by('-id')[0:1].get()
+        self.assertEquals(Message.objects.count(), 1)
+        self.assertEquals(GroupedMessage.objects.count(), 1)
+        last = Message.objects.get()
         self.assertEquals(last.logger, 'root')
         self.assertEquals(last.class_name, 'DoesNotExist')
         self.assertEquals(last.level, logging.ERROR)
@@ -411,9 +414,9 @@ class SentryTestCase(TestCase):
         else:
             self.fail('Expected an exception.')
             
-        cur = (Message.objects.count(), GroupedMessage.objects.count())
-        self.assertEquals(cur, (1, 1), 'Assumed logs failed to save. %s' % (cur,))
-        last = Message.objects.all().order_by('-id')[0:1].get()
+        self.assertEquals(Message.objects.count(), 1)
+        self.assertEquals(GroupedMessage.objects.count(), 1)
+        last = Message.objects.get()
         self.assertEquals(last.logger, 'root')
         self.assertEquals(last.class_name, 'DoesNotExist')
         self.assertEquals(last.level, logging.ERROR)
@@ -460,23 +463,79 @@ class SentryTestCase(TestCase):
     def testViewException(self):
         self.assertRaises(Exception, self.client.get, reverse('sentry-raise-exc'))
         
-        cur = (Message.objects.count(), GroupedMessage.objects.count())
-        self.assertEquals(cur, (1, 1), 'Assumed logs failed to save. %s' % (cur,))
-        last = Message.objects.all().order_by('-id')[0:1].get()
+        self.assertEquals(Message.objects.count(), 1)
+        self.assertEquals(GroupedMessage.objects.count(), 1)
+        last = Message.objects.get()
         self.assertEquals(last.logger, 'root')
         self.assertEquals(last.class_name, 'Exception')
         self.assertEquals(last.level, logging.ERROR)
         self.assertEquals(last.message, 'view exception')
         self.assertEquals(last.view, 'sentry.tests.views.raise_exc')
 
+    def testRequestMiddlwareException(self):
+        from django.conf import settings
+        orig = list(settings.MIDDLEWARE_CLASSES)
+        settings.MIDDLEWARE_CLASSES = orig + ['sentry.tests.middleware.BrokenRequestMiddleware',]
+        
+        self.assertRaises(ImportError, self.client.get, reverse('sentry'))
+        self.assertEquals(Message.objects.count(), 1)
+        self.assertEquals(GroupedMessage.objects.count(), 1)
+        last = Message.objects.get()
+        self.assertEquals(last.logger, 'root')
+        self.assertEquals(last.class_name, 'ImportError')
+        self.assertEquals(last.level, logging.ERROR)
+        self.assertEquals(last.message, 'request')
+        self.assertEquals(last.view, 'sentry.tests.middleware.process_request')
+        
+        settings.MIDDLEWARE_CLASSES = orig
+
+    # XXX: Django doesn't handle response middleware exceptions (yet)
+    # def testResponseMiddlwareException(self):
+    #     from django.conf import settings
+    #     orig = list(settings.MIDDLEWARE_CLASSES)
+    #     settings.MIDDLEWARE_CLASSES = orig + ['sentry.tests.middleware.BrokenResponseMiddleware',]
+    #     
+    #     self.assertRaises(ImportError, self.client.get, reverse('sentry'))
+    #     self.assertEquals(Message.objects.count(), 1)
+    #     self.assertEquals(GroupedMessage.objects.count(), 1)
+    #     last = Message.objects.get()
+    #     self.assertEquals(last.logger, 'root')
+    #     self.assertEquals(last.class_name, 'ImportError')
+    #     self.assertEquals(last.level, logging.ERROR)
+    #     self.assertEquals(last.message, 'response')
+    #     self.assertEquals(last.view, 'sentry.tests.middleware.process_response')
+    #     
+    #     settings.MIDDLEWARE_CLASSES = orig
+
+    def testViewMiddlewareException(self):
+        from django.conf import settings
+        orig = list(settings.MIDDLEWARE_CLASSES)
+        settings.MIDDLEWARE_CLASSES = orig + ['sentry.tests.middleware.BrokenViewMiddleware',]
+        
+        self.assertRaises(ImportError, self.client.get, reverse('sentry'))
+        self.assertEquals(Message.objects.count(), 1)
+        self.assertEquals(GroupedMessage.objects.count(), 1)
+        last = Message.objects.get()
+        self.assertEquals(last.logger, 'root')
+        self.assertEquals(last.class_name, 'ImportError')
+        self.assertEquals(last.level, logging.ERROR)
+        self.assertEquals(last.message, 'view')
+        self.assertEquals(last.view, 'sentry.tests.middleware.process_view')
+        
+        settings.MIDDLEWARE_CLASSES = orig
+
 class SentryViewsTest(TestCase):
     urls = 'sentry.tests.urls'
+    fixtures = ['sentry/tests/fixtures/views.json']
     
     def setUp(self):
         settings.DATABASE_USING = None
         self._handlers = None
         self._level = None
         settings.DEBUG = False
+        self.user = User(username="admin", email="admin@localhost", is_staff=True, is_superuser=True)
+        self.user.set_password('admin')
+        self.user.save()
     
     def tearDown(self):
         self.tearDownHandler()
@@ -509,16 +568,26 @@ class SentryViewsTest(TestCase):
         logger.setLevel(self._level)
         self._handlers = None
 
-    def testSignals(self):
-        self.assertRaises(Exception, self.client.get, '/')
-        
-        cur = (Message.objects.count(), GroupedMessage.objects.count())
-        self.assertEquals(cur, (1, 1), 'Assumed logs failed to save. %s' % (cur,))
-        last = Message.objects.all().order_by('-id')[0:1].get()
-        self.assertEquals(last.logger, 'root')
-        self.assertEquals(last.class_name, 'Exception')
-        self.assertEquals(last.level, logging.ERROR)
-        self.assertEquals(last.message, 'view exception')
+    def testTestAuth(self):
+        resp = self.client.get(reverse('sentry'), follow=True)
+        self.assertEquals(resp.status_code, 200)
+        self.assertTemplateUsed(resp, 'sentry/login.html')
+
+    def testDashboard(self):
+        self.client.login(username='admin', password='admin')
+        resp = self.client.get(reverse('sentry'), follow=True)
+        self.assertEquals(resp.status_code, 200)
+        self.assertTemplateUsed(resp, 'sentry/index.html')
+        group = resp.context['message_list'][0]
+        self.assertEquals(group.times_seen, 7)
+        self.assertEquals(group.class_name, 'AttributeError')
+        self.assertEquals(len(resp.context['message_list']), 4)
+
+    def testGroup(self):
+        self.client.login(username='admin', password='admin')
+        resp = self.client.get(reverse('sentry-group', args=[2]), follow=True)
+        self.assertEquals(resp.status_code, 200)
+        self.assertTemplateUsed(resp, 'sentry/group.html')
 
 class RemoteSentryTest(TestCase):
     urls = 'sentry.tests.urls'
@@ -567,14 +636,6 @@ class RemoteSentryTest(TestCase):
         self.assertEquals(resp.status_code, 403)
         self.assertEquals(resp.content, 'Bad data')
 
-    def testBadData(self):
-        resp = self.client.post(reverse('sentry-store'), {
-            'key': settings.KEY,
-            'data': 'hello world',
-        })
-        self.assertEquals(resp.status_code, 403)
-        self.assertEquals(resp.content, 'Bad data')
-
     def testCorrectData(self):
         kwargs = {'message': 'hello', 'server_name': 'not_dcramer.local', 'level': 40}
         data = {
@@ -604,6 +665,7 @@ class RemoteSentryTest(TestCase):
     #     self.assertEquals(instance.message, 'view exception')
     #     self.assertEquals(instance.url, 'http://testserver/?test')
     #     self.stop_test_server()
+
 class SentryFeedsTest(TestCase):
     fixtures = ['sentry/tests/fixtures/feeds.json']
     urls = 'sentry.tests.urls'
@@ -624,4 +686,85 @@ class SentryFeedsTest(TestCase):
         self.assertTrue('<link>http://testserver/</link>' in response.content)
         self.assertTrue('<title>log summaries</title>' in response.content)
         self.assertTrue('<link>http://testserver/group/1</link>' in response.content, response.content)
-        self.assertTrue('<title>(1) TypeError: TypeError: exceptions must be old-style classes or derived from BaseException, not NoneType</title>' in response.content)
\ No newline at end of file
+        self.assertTrue('<title>(1) TypeError: exceptions must be old-style classes or derived from BaseException, not NoneType</title>' in response.content)
+
+class SentryMailTest(TestCase):
+    fixtures = ['sentry/tests/fixtures/mail.json']
+    urls = 'sentry.tests.urls'
+    
+    def setUp(self):
+        settings.ADMINS = ((getpass.getuser(), '%s@localhost' % getpass.getuser()),)
+    
+    def test_mail_admins(self):
+        group = GroupedMessage.objects.get()
+        self.assertEquals(len(mail.outbox), 0)
+        group.mail_admins(fail_silently=False)
+        self.assertEquals(len(mail.outbox), 1)
+
+        out = mail.outbox[0]
+
+        self.assertTrue('Traceback (most recent call last):' in out.body)
+        self.assertTrue("COOKIES:{'commenter_name': 'admin'," in out.body)
+        self.assertEquals(out.subject, '[Django] Error (EXTERNAL IP): /group/1')
+
+    def test_mail_on_creation(self):
+        settings.MAIL = True
+        
+        self.assertEquals(len(mail.outbox), 0)
+        self.assertRaises(Exception, self.client.get, reverse('sentry-raise-exc'))
+        self.assertEquals(len(mail.outbox), 1)
+        self.assertRaises(Exception, self.client.get, reverse('sentry-raise-exc'))
+        self.assertEquals(len(mail.outbox), 1)
+
+        out = mail.outbox[0]
+
+        self.assertTrue('Traceback (most recent call last):' in out.body)
+        self.assertTrue("<Request" in out.body)
+        self.assertEquals(out.subject, '[Django] Error (EXTERNAL IP): /trigger-500')
+
+    def test_mail_on_duplication(self):
+        settings.MAIL = True
+        
+        self.assertEquals(len(mail.outbox), 0)
+        self.assertRaises(Exception, self.client.get, reverse('sentry-raise-exc'))
+        self.assertEquals(len(mail.outbox), 1)
+        self.assertRaises(Exception, self.client.get, reverse('sentry-raise-exc'))
+        self.assertEquals(len(mail.outbox), 1)
+        # XXX: why wont this work
+        # group = GroupedMessage.objects.update(status=1)
+        group = GroupedMessage.objects.all().order_by('-id')[0]
+        group.status = 1
+        group.save()
+        self.assertRaises(Exception, self.client.get, reverse('sentry-raise-exc'))
+        self.assertEquals(len(mail.outbox), 2)
+        self.assertRaises(Exception, self.client.get, reverse('sentry-raise-exc'))
+        self.assertEquals(len(mail.outbox), 2)
+
+        out = mail.outbox[1]
+
+        self.assertTrue('Traceback (most recent call last):' in out.body)
+        self.assertTrue("<Request" in out.body)
+        self.assertEquals(out.subject, '[Django] Error (EXTERNAL IP): /trigger-500')
+
+class SentryHelpersTest(TestCase):
+    def test_get_db_engine(self):
+        from django.conf import settings
+        from sentry.helpers import get_db_engine
+        _databases = getattr(settings, 'DATABASES', {}).copy()
+        _engine = settings.DATABASE_ENGINE
+        
+        settings.DATABASE_ENGINE = ''
+        settings.DATABASES['default'] = {'ENGINE': 'blah.sqlite3'}
+        
+        self.assertEquals(get_db_engine(), 'sqlite3')
+
+        settings.DATABASE_ENGINE = 'mysql'
+
+        self.assertEquals(get_db_engine(), 'sqlite3')
+
+        settings.DATABASES['default'] = {'ENGINE': 'blah.mysql'}
+
+        self.assertEquals(get_db_engine(), 'mysql')
+        
+        settings.DATABASES = _databases
+        settings.DATABASE_ENGINE = _engine
\ No newline at end of file
